/* ==========================================================================*/
/* - - - - - - - - - - - - - -  U s a r t . C   - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2010-05-29   M. Wheatley                                       */
/*	Modified  2010-08-01   MW                                                */
/*.........................................................................  */
/* WSPR Beacon   AT91SAM7256 usart handler                                    */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "usart.h"
#include "irq.h"
#include "si5338.h"
#include "iicctrl.h"
#include <stdio.h>
#include <string.h>
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

#define QUEUE_SIZE 3

//* Standard Asynchronous Mode : 8 bits , 1 stop , no parity
#define ASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )

// define some offsets into msg buffer
#define MSG_SIZE 0			//location of msg size and type
#define MSG_TYPE 1
#define MSG_CITEM 2			//location of 16 bit CItem value
#define MSG_PARAM 4			//start of CItem parameters
#define MSG_DATAADR 2		//start of Data in Data msg
#define MSG_PARAM2 5
#define MSG_FREQ 4
#define MSG_DATAADR 2
#define MSG_EEROMADR 6

#define MAX_MSG ( (128)+ 8 )	//128 bytes plus header
#define MSG_TIMEOUT  20

static const char ID_DESC[] = "WSPR Beacon";

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
xSemaphoreHandle xUsartRxSemaphore = NULL;
uint8_t UsartRxQueue[USARTQSIZE];
uint8_t UsartTxQueue[USARTQSIZE];
volatile uint32_t UsartRxHead;
uint32_t UsartRxTail;
volatile uint32_t UsartTxTail;
uint32_t UsartTxHead;

static uint32_t MessageTimer;
static uint32_t CurrentMsgPos;
static uint8_t Msg[MAX_MSG];

static BOOL SendResp;

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void UsartTask( void *pvParmeters );
static void ParseHostMsg(uint8_t* pMsg);
static void AssembleMsg( uint8_t Buf );

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        --------------------                               */
/*                       |  U s a r t I n i t |                              */
/*                        --------------------                               */
/*    Called to initialize the USART 0 port.                                 */
/*  Rx and Tx done using DMA.                                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartInit( uint32_t Rate )
{
	UsartRxHead = 0;
	UsartRxTail = 0;
	UsartTxHead = 0;
	UsartTxTail = 0;
#if REV1_PCB
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
	//* Configure uart PIO controllers to periph mode
	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
		((unsigned int) RXD0    ) |
		((unsigned int) TXD0    ),// Peripheral A
		0); // Peripheral B
	AT91C_BASE_US0->US_IDR = 0xFFFFFFFF;
	AT91C_BASE_US0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;
	AT91C_BASE_US0->US_CR = AT91C_US_RSTSTA;
	AT91C_BASE_US0->US_BRGR = ((MCK*10)/(16*Rate) + 5) /10;	//use nearest integer;
	AT91C_BASE_US0->US_MR = ASYNC_MODE;
	// Enable usart
	AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

	//* Config Usart interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US0, USART0_INTERRUPT_LEVEL,
			AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Usart0IrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US0);

	*AT91C_US0_IER = AT91C_US_RXRDY;
#else
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_SYS ) ;//enable debug unit clock
	AT91F_PIO_CfgPeriph(
		AT91C_BASE_PIOA, // PIO controller base address
		((unsigned int) DTXD    ) |
		((unsigned int) DRXD    ), // Peripheral A
		0); // Peripheral B
	AT91C_BASE_DBGU->DBGU_IDR = 0xFFFFFFFF;
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RSTSTA;
	AT91C_BASE_DBGU->DBGU_BRGR = ((MCK*10)/(16*Rate) + 5) /10;	//use nearest integer;
	AT91C_BASE_DBGU->DBGU_MR = ASYNC_MODE;
	// Enable usart
	AT91C_BASE_DBGU->DBGU_CR = AT91C_US_RXEN | AT91C_US_TXEN;
 //* Config Debug Usart interrupt(is hooked into System interrupt handler (PortISR.c)
	*AT91C_DBGU_IER = AT91C_US_RXRDY;
#endif
	xTaskCreate( UsartTask, ( const signed portCHAR * const ) "USART", 
				configMINIMAL_STACK_SIZE, NULL, USART_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | U s a r t T a s k |                                 */
/*                      -------------------                                  */
/*    Task that process usart rx bytes                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void UsartTask( void *pvParmeters )
{
 	vSemaphoreCreateBinary( xUsartRxSemaphore );
 	xSemaphoreTake( xUsartRxSemaphore, ( portTickType )10 );
 	CurrentMsgPos = 0;
    for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xUsartRxSemaphore, ( portTickType )100 ) == pdTRUE )
		{	//Bytes ready from usart
			MessageTimer = MSG_TIMEOUT;
			while( UsartRxHead != UsartRxTail)
			{
				AssembleMsg(UsartRxQueue[UsartRxTail++]);
				if(UsartRxTail >= USARTQSIZE)
					UsartRxTail = 0;
			}
		}
		else
		{
			if( MessageTimer )
			{
				MessageTimer--;
				if( !MessageTimer )
				{
					CurrentMsgPos = 0;
				}
			}
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | S e n d U s a r t M s g |                           */
/*                      -------------------------                            */
/*    Function that sends a Msg buffer of length Len to the usart            */
/* Dont call from IRQ or ka-boom.                                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUsartMsg(uint8_t* Buf, uint32_t Len)
{
int i;
#if REV1_PCB
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	for(i=0; i<Len; i++)
	{
		UsartTxQueue[UsartTxHead++] = Buf[i];
		UsartTxHead &= (USARTQSIZE-1);
	}
	*AT91C_US0_IER = AT91C_US_TXRDY;
#else
	*AT91C_DBGU_IDR = AT91C_US_TXRDY;
	for(i=0; i<Len; i++)
	{
		UsartTxQueue[UsartTxHead++] = Buf[i];
		UsartTxHead &= (USARTQSIZE-1);
	}
	*AT91C_DBGU_IER = AT91C_US_TXRDY;
#endif
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | A s s e m b l e M s g |                             */
/*                      -----------------------                              */
/*    Routine that manages receiving ASCP format message bytes               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void AssembleMsg( uint8_t ch )
{
	Msg[CurrentMsgPos++] = ch;	//get next msg byte
	if(CurrentMsgPos >= MAX_MSG)
	{
		CurrentMsgPos = 0;
	}
	else if( CurrentMsgPos == Msg[MSG_SIZE] )
	{
		CurrentMsgPos = 0;	//got a complete msg
		ParseHostMsg(Msg);
	}
	else if( CurrentMsgPos > Msg[MSG_SIZE] )
	{
		CurrentMsgPos = 0;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -------------------------                              */
/*                   | P a r s e H o s t M s g |                             */
/*                    -------------------------                              */
/*    This routine is called to parse a new message from the Host.           */
/* INPUTS: The Msg Buffer 'pMsg' containing the new msg from the Host        */
/*  used for jumping into boot code for code update                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ParseHostMsg(uint8_t* pMsg)
{
uint32_t i;
uint32_t mType;
uint32_t mCitem;
uint32_t mLength;
t32BITS tmp;
uint8_t ch;
	mLength = (uint32_t)pMsg[MSG_SIZE+1];		//get length
	mLength <<= 8;
	mLength |= (uint32_t)pMsg[MSG_SIZE];
	mLength &= LENGTH_MASK;
	mType = (uint32_t)(pMsg[MSG_TYPE] & 0xE0);	//strip out just type field
	mCitem = (uint32_t)pMsg[MSG_CITEM+1];		//get CItem
	mCitem <<= 8;
	mCitem |= (uint32_t)pMsg[MSG_CITEM];
	SendResp = TRUE;

//printf("len = 0x%X  ", mLength);
//printf("msg = 0x%X  ", mCitem);
//printf("dat0 = 0x%X ", pMsg[MSG_PARAM+0]);
//printf("dat1 = 0x%X\r\r\n\n", pMsg[MSG_PARAM+1]);


	if( mType == TYPE_HOST_SET_CITEM )
	{	// Is a message from host to set a control item value
		pMsg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch( mCitem )
		{
			case CI_UPDATE_MODE_CONTROL:
				if(pMsg[MSG_PARAM] == 0)	//if a channel 0 (bootloader cmd)
				{
					if( (pMsg[MSG_PARAM+2] == PRODUCT_ID[0] ) &&
						(pMsg[MSG_PARAM+3] == PRODUCT_ID[1] ) &&
						(pMsg[MSG_PARAM+4] == PRODUCT_ID[2] ) &&
						(pMsg[MSG_PARAM+5] == PRODUCT_ID[3] ) )
					{	//is valid boot command
						if( pMsg[MSG_PARAM+1] == CI_UPDATE_MODE_CONTROL_ENTER )
						{	//enter boot mode via a software induced reset
							pMsg[MSG_SIZE] = CI_UPDATE_MODE_CONTROL_SETRESPLEN;
							SendUsartMsg(pMsg, (uint32_t)pMsg[MSG_SIZE] );	//ack msg first
							AT91F_PIO_SetOutput( AT91C_BASE_PIOA, GPSTOG );
							vTaskDelay(100);
							AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, GPSTOG );
							while( (AT91C_BASE_US0->US_CSR & AT91C_US_TXEMPTY) == 0 )
								vTaskDelay( 10 );	//wait for ack to be sent
							taskDISABLE_INTERRUPTS();
							RfOff();
							*AT91C_US0_CR = AT91C_US_TXDIS | AT91C_US_RXDIS;
							*AT91C_US0_PTCR = AT91C_PDC_RXTDIS|AT91C_PDC_TXTDIS;
							*AT91C_US0_IDR = ~(0);
							*AT91C_RSTC_RCR = ((unsigned int) 0xA5 << 24)|AT91C_RSTC_PROCRST|AT91C_RSTC_PERRST;	// SW reset back to boot code
						}
						else
						{
							pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK
							pMsg[MSG_TYPE] = 0;
						}
					}
					else
					{
						pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK
						pMsg[MSG_TYPE] = 0;
					}
				}
				break;
			case CI_TX_FREQUENCY:	//set xmit center frequency
				pMsg[MSG_SIZE] = CI_TX_FREQUENCY_SETRESPLEN;
				break;
			case CI_OPMODE:	//set Operational Mode
				pMsg[MSG_SIZE] = CI_OPMODE_SETRESPLEN;
				SetOpMode(pMsg[MSG_PARAM+0]);
				WriteFRAM(EE_OPMODE, Settings.OpMode);
				break;
			default:
				pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK
				pMsg[MSG_TYPE] = 0;
				break;
		}
	}
	else if( mType == TYPE_HOST_REQ_CITEM )
	{	// Is a message from host requesting a control item value
		pMsg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch(mCitem)
		{
			case CI_GENERAL_INTERFACE_NAME:
				i = 0;
				do{
					ch = ID_DESC[i];
					pMsg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<20) );
				pMsg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_INTERFACE_SERIALNUM:
				i = 0;
				do{
					ch = SERIAL_NUMBER[i];
					pMsg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<9) );
				pMsg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_INTERFACE_VERSION:
				pMsg[MSG_SIZE] = CI_GENERAL_INTERFACE_VERSION_SETRESPLEN;
				pMsg[MSG_PARAM] = (VB_INTERFACE_VERSION&0x00FF);
				pMsg[MSG_PARAM+1] = ((VB_INTERFACE_VERSION>>8)&0x00FF);
				break;
			case CI_GENERAL_HARDFIRM_VERSION:
				pMsg[MSG_SIZE] = CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN;
				if(pMsg[MSG_PARAM+0] == 0)	//boot code version
				{
					pMsg[MSG_PARAM+1] = *BOOT_VERSION & 0x00FF;
					pMsg[MSG_PARAM+2] = (*BOOT_VERSION>>8) & 0x00FF;
				}
				else if(pMsg[MSG_PARAM+0] == 1) //flash version
				{
					pMsg[MSG_PARAM+1] =  FIRMWARE_VERSION & 0x00FF;
					pMsg[MSG_PARAM+2] =  (FIRMWARE_VERSION>>8) & 0x00FF;
				}
				else if(pMsg[MSG_PARAM+0] == 2) 	//HW version
				{
					pMsg[MSG_PARAM+1] = *HW_VERSION&0x00FF;
					pMsg[MSG_PARAM+2] = (*HW_VERSION>>8)&0x00FF;
				}
				break;
			case CI_GENERAL_STATUS_CODE:
				pMsg[MSG_SIZE] = CI_GENERAL_STATUS_CODE_SETRESPLEN;
				pMsg[MSG_PARAM] = GENERAL_STATUS_STOPPED;
				break;
			case CI_UPDATE_MODE_PARAMS:
				pMsg[MSG_SIZE] = CI_UPDATE_MODE_PARAMS_SETRESPLEN;
				tmp.all = FLASH_SIZE;
				pMsg[MSG_PARAM+1] = tmp.bytes.b0;
				pMsg[MSG_PARAM+2] = tmp.bytes.b1;
				pMsg[MSG_PARAM+3] = tmp.bytes.b2;
				pMsg[MSG_PARAM+4] = tmp.bytes.b3;
				tmp.all = FLASH_PAGE_SIZE;
				pMsg[MSG_PARAM+5] = tmp.bytes.b0;
				pMsg[MSG_PARAM+6] = tmp.bytes.b1;
				pMsg[MSG_PARAM+7] = tmp.bytes.b2;
				pMsg[MSG_PARAM+8] = tmp.bytes.b3;
				tmp.all = FLASH_SECTOR_SIZE;
				pMsg[MSG_PARAM+9] = tmp.bytes.b0;
				pMsg[MSG_PARAM+10] = tmp.bytes.b1;
				pMsg[MSG_PARAM+11] = tmp.bytes.b2;
				pMsg[MSG_PARAM+12] = tmp.bytes.b3;
				break;
			case CI_TX_FREQUENCY:	//get xmit freq
				pMsg[MSG_SIZE] = CI_TX_FREQUENCY_SETRESPLEN;
				break;
			case CI_OPMODE:	//get Operationsl Mode
				pMsg[MSG_SIZE] = CI_OPMODE_SETRESPLEN;
				pMsg[MSG_PARAM] = Settings.OpMode;
				break;
			case CI_GENERAL_PRODUCT_ID:
				pMsg[MSG_SIZE] = CI_GENERAL_PRODUCT_ID_SETRESPLEN;
				pMsg[MSG_PARAM+0] = PRODUCT_ID[0];	//use values from boot rom
				pMsg[MSG_PARAM+1] = PRODUCT_ID[1];
				pMsg[MSG_PARAM+2] = PRODUCT_ID[2];
				pMsg[MSG_PARAM+3] = PRODUCT_ID[3];
				break;
			default:
//				printf("nack CI= 0x%X\r\n", mCitem);
				pMsg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK
				pMsg[MSG_TYPE] = 0;
				break;
		}
	}
	else if( mType == TYPE_HOST_DATA_ITEM0 )
	{
		SendResp = FALSE;
	}
	else if( mType == TYPE_HOST_DATA_ITEM1 )
	{
		SendResp = FALSE;
	}
	else if( mType == TYPE_HOST_DATA_ITEM2 )
	{
		SendResp = FALSE;
	}
	else if( mType == TYPE_HOST_DATA_ITEM3 )
	{
		SendResp = FALSE;
	}
	else if(mType == TYPE_DATA_ITEM_ACK)
	{
		SendResp = FALSE;
	}
	else
	{
		SendResp = FALSE;
	}
	if(	SendResp )
	{
		SendUsartMsg(pMsg, (uint32_t)pMsg[MSG_SIZE] );
		SendResp = FALSE;
	}
}


void SetOpMode(uint8_t Mode)
{
	Settings.OpMode = Mode;
	switch(Mode)
	{
		case CI_OPMODE_SETUP:
			break;
		case CI_OPMODE_TUNE:
			break;
		case CI_OPMODE_WSPR:
			break;
		case CI_OPMODE_GPS:
			break;
		case CI_OPMODE_GPSRAW:
			break;
	}
}

