/* ==========================================================================*/
/* - - - - - - - - - - - - -  W s p r . c  - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Implementation File for the wspr module                                */
/*	Created   2011-04-23   M. Wheatley                                       */
/*	Modified  2011-04-23   MW                                                */
/* WSPR Beacon   AT91SAM7256 wispr  gen                                      */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "wspr.h"
#include "si5338.h"
#include "gps.h"
#include "iicctrl.h"
#include "timer.h"
#include "adctrl.h"
#include "protocoldefs.h"
#include "wsprencoder.h"
#include <stdlib.h>
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
xQueueHandle xWsprCmdQueue = NULL;
uint32_t SymbCount;
uint32_t TotalTransmissions;
int MinuteCounter;
BOOL Running;
BOOL TxError;

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void WsprTask( void *pvParmeters );
static void StartBitTimer();
static void StopBitTimer();
static void SendSymbol();

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ------------------                                 */
/*                       |  W s p r I n i t |                                */
/*                        ------------------                                 */
/*    Called to initialize the WSPR generator                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WsprInit( )
{
	StopBitTimer();
	xWsprCmdQueue = xQueueCreate( 5, sizeof( uint8_t ) );

	xTaskCreate( WsprTask, ( const signed portCHAR * const ) "WSPR",
				configMINIMAL_STACK_SIZE, NULL, WSPR_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | W s p r T a s k |                                   */
/*                      -----------------                                    */
/*    Task that process WSPR events                                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void WsprTask( void *pvParmeters )
{
uint8_t Cmd;
	SymbCount = 0;
	MinuteCounter = 0;
	Running = FALSE;
	TxError = FALSE;
	TotalTransmissions = 0;
    for( ;; )	//loop forever in this task
    {
        if( xQueueReceive( xWsprCmdQueue, &Cmd, ( portTickType ) 250 ) )
		{	//process new cmd from GPS or bit timer
			switch(Cmd)
			{
				case WPSRCMD_BIT:	//from WSPR bit timer to send next symbol
					if(Running)
					{
						SendSymbol();	//get and send next WSPR symbol
						//toggle red led
						if( RLED == (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & RLED) )
							SetRLedPWM( LED_OFF );
						else
							SetRLedPWM( LED_ON );
						if(GetVSWR() > 500)	//check VSWR to see if ok every symbol time
						{
							TxError = TRUE;
							printf("\r\n\r\nVSWR greater than 5!!\r\n\r\n");
						}
						if( TxError || (CI_OPMODE_WSPR != Settings.OpMode)  )
						{	//abort transmission if mode changed while running or VSWR > 5 to 1
							RfOff();
							StopBitTimer();
							SymbCount = 0;
							Running = FALSE;
						}
					}
					break;
				case WPSRCMD_SECOND:
					if(!Running && TimeIsAccurate && ( CI_OPMODE_WSPR == Settings.OpMode) )
					{	//if not running check to see if time to start next msg
						//see if at top of minute
						if( 0==SecondTimer )
						{
							if(MinuteCounter)
								MinuteCounter--;
							if( 0==(MinuteTimer&1) )
							{	//here on every even minute
								if(MinuteCounter <= 0)
								{
									if(WsprEncode(Settings.CallSign, (char*)MHStr, Settings.WsprPower))
									{
										RfOn();
										SymbCount = 0;
										StartBitTimer();
										Running = TRUE;
										SendSymbol();	//start first symbol
										if(0 == Settings.WsprInterval)
											MinuteCounter = 0;		//no interval
										else
											MinuteCounter = Settings.WsprInterval + (rand() & 2);
										TxError = FALSE;
									}
									else
									{
										printf("\r\nFailed to Encode WSPR Data!!\r\n");
										MinuteCounter = 0;
									}
								}
							}
						}
						else
						{	//here every second except at top of minute and not running
							//blip red led if waiting to start
							AT91F_PIO_SetOutput( AT91C_BASE_PIOA, RLED );
							vTaskDelay(5);
							AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, RLED );
							if( (0==MinuteCounter) && ( 0==(MinuteTimer&1) ) )
								MinuteCounter = 1;
						}
					}
					break;
				default:
					break;
			}
		}
    }//end for
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   ---------------------                                   */
/*                  | S e n d S y m b o l |                                  */
/*                   ---------------------                                   */
/*    Send next WSPR symbol                                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void SendSymbol()
{
uint8_t symb;
	symb = Symbols[SymbCount];	//get next symbol to send
	SetWsprSymbFreq( (int32_t)(symb&0x03) );	//set new symbol freq
	SymbCount++;
	if(SymbCount > NUM_SYMBOLS)
	{	//stop if done sending all the msg symbols
		RfOff();
		StopBitTimer();
		SymbCount = 0;
		Running = FALSE;
		TotalTransmissions++;
	}
//	printf("    \r%d %d\r",SymbCount, symb);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   ---------------------------                             */
/*                  | S t a r t B i t T i m e r |                            */
/*                   ---------------------------                             */
/*    Start bit timer                                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void StartBitTimer()
{
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC0);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------                               */
/*                  | S t o p B i t T i m e r |                              */
/*                   -------------------------                               */
/*    Stop bit timer                                                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void StopBitTimer()
{
	AT91F_AIC_DisableIt (AT91C_BASE_AIC, AT91C_ID_TC0);

}


