#ifndef COMMON_H_
#define COMMON_H_
#include <stdio.h>
#include <ctype.h>
#include <stdint.h>

/* ==========================================================================*/
/* - - - - - - - - - - - C o m m o n . h   - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    common Definition File for the program                                 */
/*  Created   2005-11-03	msw                                              */
/*  Modified  2012-02-11    msw   changed to standard stdint.h data types    */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

#define FIRMWARE_VERSION 100	//firmware revision * 100 ( 123 -> 1.23 )

#define true	-1
#define false	0
#define TRUE	-1
#define FALSE	0

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

typedef int BOOL;		//

typedef union			//highly non portable way to access bytes of int
{
	struct b4
	{
		uint8_t b0;
		uint8_t b1;
		uint8_t b2;
		uint8_t b3;
	}bytes;
	struct uw2
	{
		uint16_t w0;
		uint16_t w1;
	}words;
	uint32_t all;
	uint8_t ball[4];
}t32BITS;

typedef union			//highly non portable way to access bytes of short int
{
	struct b2
	{
		uint8_t b0;
		uint8_t b1;
	}bytes;
	uint16_t all;
	uint8_t ball[2];
}t16BITS;

typedef union			//highly non portable way to access bytes of short int
{
	struct b3
	{
		uint8_t b0;
		uint8_t b1;
	}bytes;
	int16_t all;
	uint8_t ball[2];
}tS16BITS;

typedef union
{
	struct w2
	{
		uint16_t w0;
		uint16_t w1;
	}word;
	uint32_t all;
}t32Words;

/* 
   RAMFUNC 
   Attribute which defines a function to be located 
   in memory section .fastrun and called via "long calls". 
   See linker-skript and startup-code to see how the
   .fastrun-section is handled.
*/
#define RAMFUNC __attribute__ ((long_call, section (".fastrun")))


#define htons(A) ((((uint16_t)(A) & 0xff00) >> 8) | \
                    (((uint16_t)(A) & 0x00ff) << 8))
#define htonl(A) ((((uint32_t)(A) & 0xff000000) >> 24) | \
                    (((uint32_t)(A) & 0x00ff0000) >> 8)  | \
                    (((uint32_t)(A) & 0x0000ff00) << 8)  | \
                    (((uint32_t)(A) & 0x000000ff) << 24))

#define ntohs  htons
#define ntohl  htonl

#endif /*COMMON_H_*/
