/* ==========================================================================*/
/* - - - - - - - - - - - - -  i i c c t r l . h  - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Include File for the iicctrl.c module                                  */
/*  Created   2010-08-01	msw                                              */
/*  Modified  2010-08-01    msw                                              */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#ifndef IICCTRL_H_
#define IICCTRL_H_

#include "common.h"
/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/*---------------------------------------------------------------------------*/
/*--------------------> G L O B A L   D E F I N E S <------------------------*/
/*---------------------------------------------------------------------------*/
#define FREQ_MIN 0
#define FREQ_MAX 200
#define FREQ_DEF 100

#define INTERVAL_MIN 0
#define INTERVAL_MAX 200
#define INTERVAL_DEF 8

#define POWER_MIN 0
#define POWER_MAX 60
#define POWER_DEF 33

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/

#define WSPR_BW	200 //all bands are 200Hz wide

#define BANDINDEX_160 0
#define BANDINDEX_80 1
#define BANDINDEX_40 2
#define BANDINDEX_30 3
#define BANDINDEX_20 4
#define BANDINDEX_17 5
#define BANDINDEX_15 6
#define BANDINDEX_12 7
#define BANDINDEX_10 8
#define BANDINDEX_DEF 9

//EEROM address assignments (Keep order same as in Settings Structure below!)
#define EE_FREQOFFSET 0		//(0-39) 4 bytes * 10 saved frequency offsets for the bands
#define EE_OPMODE 40 		//(40) 1 byte Operation Mode
#define EE_WSPRINTERVAL 41	//(41) 1 byte xmit interval
#define EE_WSPRPOWER 42		//(42) 1 byte xmit power in dBm
#define EE_CALLSIGN 43		//(43-52) 10 byte callsign string

#define EE_SIZE 53			//Size of EEROM items (manual update if change above defs)

typedef struct
{
	uint32_t FreqOffset[10];
	uint8_t BandIndex;		//not saved in eerom but set by resistor values
	uint8_t OpMode;
	uint8_t WsprInterval;		//in 2 minute steps
	uint8_t WsprPower;		//in dBm
	char CallSign[10];
}tSettings;

/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    F u n c t i o n s <-------------------*/
/*---------------------------------------------------------------------------*/
void IICCtrlInit( void );
void GetSettings();
void SetFactoryDefaults();
void WriteSi( uint8_t Adr, uint8_t Data );
uint8_t ReadSi( uint8_t Adr );
uint8_t ReadIIC( uint8_t adr, uint8_t Reg );
void WriteIIC( uint8_t adr, uint8_t Reg, uint8_t Data );
void WriteFRAM( uint16_t Adr, uint8_t Data );
uint8_t ReadFRAM( uint16_t Adr );
void Write16FRAM( uint16_t Adr, uint16_t Data );
void Write32FRAM( uint16_t Adr, uint32_t Data );
uint16_t Read16FRAM( uint16_t Adr );
uint32_t Read32FRAM( uint16_t Adr );
void SaveAll();

/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    V a r i a b l e s <-------------------*/
/*---------------------------------------------------------------------------*/
extern tSettings Settings;
extern BOOL NeedToSaveSettings;
extern const uint32_t FREQBASE_TBL[];

#endif /*IICCTRL_H_*/
